package domain;

import java.util.*;

import junit.framework.*;

public class CardTest extends TestCase {
   public static final Card CARD1 = new Card(Rank.king, Suit.hearts);
   private static final Card CARD1_COPY = new Card(Rank.king, Suit.hearts);
   private static final Card CARD1_COPY2 = new Card(Rank.king, Suit.hearts);
   public static final Card CARD2 = new Card(Rank.queen, Suit.hearts);
   public static final Card CARD3 = new Card(Rank.king, Suit.diamonds);

   public void testCreate() {
      Card card = new Card(Rank.ace, Suit.clubs);
      assertEquals(Rank.ace, card.getRank());
      assertEquals(Suit.clubs, card.getSuit());
   }

   public void testEquality() {
      assertTrue(CARD1.equals(CARD1_COPY));
      assertFalse(CARD1.equals(CARD2));
      assertFalse(CARD1.equals(CARD3));
      Card card1subtype = new Card(Rank.king, Suit.hearts) {};
      assertFalse(CARD1.equals(card1subtype));

      assertReflexivity();
      assertSymmetry();
      assertTransitivity();
      assertConsistency();
      assertNullComparison();
   }
   
   public void testHashCode() {
      assertHashCodeConsistency();
      assertHashCodeEquality();
      // assertHashCodePerformance();
   }
   
//   private void assertHashCodePerformance() {
//      final int capacity = 100000;
//      final int iterations = capacity * 8 / 10 / 52;
//      Set<Card> set = new HashSet<Card>(capacity);
//
//      long start = System.currentTimeMillis();
//      for (int i = 0; i < iterations; i++)
//         for (Suit suit: Suit.values())
//            for (Rank rank: Rank.values())
//               set.add(new Card(rank, suit));
//      long stop = System.currentTimeMillis();
//      long elapsed = stop - start;
//      assertTrue("elapsed: " + elapsed, elapsed < 100);
//   }

   private void assertHashCodeEquality() {
      assertEquals(CARD1.hashCode(), CARD1_COPY.hashCode());
   }

   private void assertHashCodeConsistency() {
      assertEquals(CARD1.hashCode(), CARD1.hashCode());
   }

   private void assertNullComparison() {
      assertFalse(CARD1.equals(null));
   }

   private void assertConsistency() {
      assertTrue(CARD1.equals(CARD1_COPY));
      assertFalse(CARD1.equals(CARD2));
   }

   private void assertTransitivity() {
      assertTrue(CARD1_COPY.equals(CARD1_COPY2));
      assertTrue(CARD1.equals(CARD1_COPY2));
   }

   private void assertSymmetry() {
      assertTrue(CARD1.equals(CARD1_COPY));
      assertTrue(CARD1_COPY.equals(CARD1));
   }

   private void assertReflexivity() {
      assertTrue(CARD1.equals(CARD1));
   }

   public static void assertCards(List<Card> cards, Card... expected) {
      assertEquals(expected.length, cards.size());
      int i = 0;
      for (Card card: expected)
         assertEquals(card, cards.get(i++));
   }
}