package domain;

import java.util.*;

public class Deck {
   public static final int SIZE = 52;
   private List<Card> cards = new LinkedList<Card>();
   private Iterator<Card> it;

   public Deck() {
      for (Suit suit: Suit.values())
         for (Rank rank: Rank.values())
            cards.add(new Card(rank, suit));
      Collections.shuffle(cards);
   }

   public int cardsRemaining() {
      return cards.size();
   }
   
   public boolean contains(Rank rank, Suit suit) {
      return cards.contains(new Card(rank, suit));
   }

   public Card deal() {
      if (cardsRemaining() == 0)
         throw new EmptyDeckException();
      if (it == null)
         it = cards.iterator();
      Card card = it.next();
      it.remove();
      return card;
   }

   // primarily used for testing
   Card top() {
      return cards.get(0);
   }

   // primarily used for testing
   public Card[] top(int count) {
      Card[] results = new Card[count];
      for (int i = 0; i < count; i++)
         results[i] = cards.get(i);
      return results;
   }
}
