package domain;

import java.util.*;

import junit.framework.*;

public class DeckTest extends TestCase {
   private Deck deck1;

   protected void setUp() {
      deck1 = new Deck();
   }
   
   public void testCreate() {
      assertEquals(Deck.SIZE, deck1.cardsRemaining());
      for (Suit suit: Suit.values())
         for (Rank rank: Rank.values())
            assertTrue(deck1.contains(rank, suit));
   }
   
   public void testDeal() {
      List<Card> hand = createDeckSizedHand();
      assertDealAll(deck1, hand);
   }
   
   private List<Card> createDeckSizedHand() {
      return new ArrayList<Card>(Deck.SIZE);
   }

   public void testDeckShuffledOnCreation() {
      Deck deck2 = new Deck();
      List<Card> hand1 = createDeckSizedHand(); 
      List<Card> hand2 = createDeckSizedHand();
      assertDealAll(deck1, hand1);
      assertDealAll(deck2, hand2);
      assertFalse(hand1.equals(hand2));
   }
   
   public void testDealTooMany() {
      assertDealAll(deck1, new ArrayList<Card>());
      try {
         deck1.deal();
         fail("should have received exception");
      }
      catch (EmptyDeckException expected) {
         assertEquals(0, deck1.cardsRemaining());
      }
   }
   
   // seeing the top card is very valuable in testing
   public void testTop() {
      Card top = deck1.top();
      assertEquals(Deck.SIZE, deck1.cardsRemaining());
      Card card = deck1.deal();
      assertEquals(top, card);
   }

   // seeing the top N cards is very valuable in testing
   public void testTopN() {
      Card[] top = deck1.top(3);
      assertEquals(Deck.SIZE, deck1.cardsRemaining());
      assertEquals(3, top.length);
      assertEquals(top[0], deck1.deal());
      assertEquals(top[1], deck1.deal());
      assertEquals(top[2], deck1.deal());
   }
   
   private void assertDeal(Deck deck, Card card, int i) {
      assertEquals(i - 1, deck.cardsRemaining());
      assertFalse(deck.contains(card.getRank(), card.getSuit()));
   }

   private void assertDealAll(Deck deck, List<Card> hand) {
      for (int i = Deck.SIZE; i > 0; i--) {
         Card card = deck.deal();
         assertDeal(deck, card, i);
         hand.add(card);
      }
      assertEquals(0, deck.cardsRemaining());
   }

   public static void assertCardsDealt(Deck deck, Card... cards) {
      for (Card card: cards)
         assertFalse(deck.contains(card.getRank(), card.getSuit()));
   }
}