package domain;

import java.util.*;

public class Game {
   public static final int CAPACITY = 10;
   private List<Player> players = new ArrayList<Player>();
   private List<Card> community = new ArrayList<Card>();
   private Deck deck = new Deck();
   private int button = 1;
   private int smallAmount;
   private int bigAmount;

   public List<Player> players() {
      return players;
   }

   public void add(Player player) {
      players.add(player);
   }

   // needed for testing 
   Deck deck() {
      return deck;
   }

   public void dealFlop() {
      burn();
      for (int i = 0; i < 3; i++)
         community.add(deck.deal());
   }

   private void burn() {
      deck.deal();
   }

   public List<Card> community() {
      return community;
   }

   public void dealTurn() {
      burnAndTurn();
   }

   public void dealRiver() {
      burnAndTurn();
   }

   private void burnAndTurn() {
      burn();
      community.add(deck.deal());
   }

   public void setButton(int i) {
      button = i;
   }

   public void setBlinds(int small, int big) {
      this.smallAmount = small;
      this.bigAmount = big;
   }

   public void startHand() {
      collectBlinds();
   }

   public int buttonPosition() {
      return button;
   }

   public void stopHand() {
      removeAllCards();
      advanceButton();
   }

   private void removeAllCards() {
      for (Player player: players())
         player.removeCards();
      community.clear();
   }

   public void dealHoleCards() {
      for (int round = 0; round < 2; round++) {
         for (int i = 1; i <= players.size(); i++) {
            Player player = getPlayer(button + i);
            player.dealToHole(deck.deal());
         }
      }
   }

   private void collectBlinds() {
      Player small = getPlayer(button + 1);
      Player big = getPlayer(button + 2);
      small.bet(smallAmount);
      big.bet(bigAmount);
   }

   private void advanceButton() {
      button = ringPosition(button + 1);
   }

   private int ringPosition(int position) {
      if (position > players.size())
         return position - players.size();
      return position;
   }

   private Player getPlayer(int position) {
      return players.get(ringPosition(position) - 1);
   }
}