package domain;

import java.util.*;

import junit.framework.*;

public class PlayerTest extends TestCase {
   private final static String NAME = "x";
   private Player player;

   protected void setUp() {
      player = new Player(NAME);
   }

   public void testCreate() {
      assertEquals(NAME, player.getName());
   }
   
   public void testHoleCards() {
      player.dealToHole(CardTest.CARD1);
      player.dealToHole(CardTest.CARD2);
      List<Card> holeCards = player.holeCards();
      CardTest.assertCards(holeCards, CardTest.CARD1, CardTest.CARD2);
   }
   
   public void testRemoveCards() {
      player.dealToHole(CardTest.CARD1);
      player.dealToHole(CardTest.CARD2);
      player.removeCards();
      assertTrue(player.holeCards().isEmpty());
   }
   
   public void testBankroll() {
      assertEquals(0, player.chipCount());
      player.add(1000);
      assertEquals(1000, player.chipCount());
      player.bet(200);
      assertEquals(800, player.chipCount());
   }
   
   public void testInsufficientFunds() {
      try {
         player.bet(1);
         fail("expected insuff. funds exception");
      }
      catch (InsufficientFundsException expected) {
         assertEquals(0, player.chipCount());
      }
   }
}